;;; attach a reactor to dimensions that hides the lower tolerance limit
;;; 11/2018 knut hohenberg
;;; no rights reserved - no responsibility taken!

(defun KHAD:hide_lower_tolerance (dimnme / 
                            dimstl dimeed dimpost dimdsep dimlfac dimtfac dimmzs dimmzf 
                            dimdat bit70 blknme object objdat dimtxt n prefix dimval suffix
                            dpost1 dpost2 m cm mm subs1 subs2
                           )
 (if (KHAD:check_dimlin dimnme)
  (progn
   (setq dimdat (entget dimnme))
   (setq dimval (cdr (assoc 1 dimdat)))
   (if (not (setq dimtad (cdr (assoc 77 (KHAD:get_dimoverrides dimnme)))))
    (setq dimtad (cdr (assoc 77 (entget (tblobjname "DIMSTYLE" (cdr (assoc 3 (entget dimnme))))))))
   )
   (setq n 0
         prefix ""
         suffix ""
   )
   (if (wcmatch dimval "*<>*") ;annotated dimension
    (while (and n (< n (strlen dimval)))
     (setq n (1+ n))
     (if (= "<" (substr dimval n 1))
      (if (= ">" (substr dimval (1+ n) 1))
       (cond
        ((= 1 n)
         (setq suffix (substr dimval (+ n 2)))
         (setq n nil)
        )
        ((< 1 n (- (strlen dimval) 1))
         (setq prefix (substr dimval 1 (1- n)))
         (setq suffix (substr dimval (+ n 2)))
         (setq n nil)
        )
        (t
         (setq prefix (substr dimval 1 (1- n)))
         (setq n nil)
        )
       )
      )
     )
    )
   )
   (if (setq n (vl-string-search "\\X" suffix));everything behind will go into a second Mtext-object
    (setq suffix (substr suffix 1 n))          ;and can be ignored here
   )
   (setq bit70 (cdr (assoc 70 dimdat)))
   (setq blknme (cdr (assoc 2 dimdat)))
   (setq object (cdr (assoc -2 (tblsearch "block" blknme))))
   (while object
    (setq object (entnext object))
    (if object (setq objdat (entget object)))
    (if (= "MTEXT" (cdr (assoc 0 objdat)))
     (setq
      dimtxt (cdr (assoc 1 objdat))
      object nil
     )
    )
   )
   (if (and (/= "" prefix) (wcmatch dimtxt (strcat prefix "*")))
    (setq dimtxt (substr dimtxt (1+ (strlen prefix))))
   )
   (if (and (/= "" suffix) (wcmatch dimtxt (strcat "*" suffix)))
    (setq dimtxt (substr dimtxt 1 (-(strlen dimtxt)(strlen suffix))))
   )
   (setq n 0)
   (setq dimtxt (last (vle-string-split "S" dimtxt))); get rid of stacking
   (setq dimtxt (car (vle-string-split "^" dimtxt))); get rid of lower tolerance value
   (cond
    ((< 0 dimtad 4)
     (setq dimtxt (strcat "\\S ^" prefix dimtxt suffix)) ; reverse stacking, otherwise text position would be wrong
    )
    ;((= 4 dimtad)
    ; (setq dimtxt (strcat "\\S" prefix dimtxt suffix "^ ")) ; maintain stacking - not giving the desired result
    ;)
    (t
     (setq dimtxt (strcat prefix dimtxt suffix)) ;remove stacking for centered / below dimline text
    )
   )
   (print dimtxt)
   (setq objdat (subst (cons 1 dimtxt) (assoc 1 objdat) objdat))
   (entmod objdat)
   (entupd dimnme)
  )
 )
)

;check if a dimension has limits display activated
(defun KHAD:check_dimlin (dimnme / dimlin)
 (if (not (setq dimlin (cdr (assoc 72 (KHAD:get_dimoverrides dimnme)))))
  (setq dimlin (cdr (assoc 72 (entget (tblobjname "DIMSTYLE" (cdr (assoc 3 (entget dimnme))))))))
 )
 (if (= 1 dimlin)
  T
  NIL
 )
)

; return a list of dimension overrides as dotted pairs
(defun KHAD:get_dimoverrides (dimnme / dimeed n dxflst dpair key value)
 (if (setq dimeed (cdadr (assoc -3 (entget dimnme '("Acad")))))
  (progn
   (setq n 2
         dxflst '()
   )
   (while n
    (setq dpair (nth n dimeed))
    (cond
     ;((= 1002 (car dpair)) (setq n NIL))
     ((= 1070 (car dpair))
      (setq
            key (cdr dpair)
            value (cdr (nth (1+ n) dimeed))
            dxflst (cons (cons key value) dxflst)
            n (+ 2 n)
      )
     )
     (T (setq n NIL))
    )
   )
   (reverse dxflst)
  )
 )
)


;;; return a given reactor, if it exists
(defun KHAD:find_reactor (type name / rlist n reactor)
 (setq rlist (car(vlr-reactors type))
       n 1
 )
 (if rlist 
  (while n
   (setq reactor (nth n rlist))
   (if reactor
    (if (= name (vlr-data reactor))
     (setq n nil)
     (setq n (+ 1 n))
    )
    (setq n nil)
   )
  )
 )
 reactor
)

;select objects attached to a reactor
(defun KHAD:select_reactor_owners (type name / selset reactor obj)
 (setq selset (ssadd)
       reactor (KHAD:find_reactor type name)
 )
 (foreach obj (vlr-owners reactor)
  (if obj
   (ssadd (vlax-vla-object->ename obj) selset)
  )
 )
)

;attach persistent formatting that hides the lower tolerance to properly configured dimensions
(defun KHAD:attach_toleranceformat ( / reactor selset n count dimnme dimdat dimtxt bit70 dimobj)
 (if (not (setq reactor (KHAD:find_reactor :vlr-object-reactor "KHAD:dimtolerancereactor")))
  (progn
   (setq reactor (vlr-object-reactor '() "KHAD:dimtolerancereactor" '((:vlr-modified . KHAD:format_tolerance))))
   (vlr-pers reactor)
  )
 )
 (setq count 0)
 (prompt "\nSelect dimension objects to format: ")
 (if (setq selset (ssget '((-4 . "<XOR")(0 . "DIMENSION")(0 . "ARC_DIMENSION")(-4 . "XOR>"))))
  (repeat (setq n (sslength selset))
   (setq dimnme (ssname selset (setq n (1- n)))
         dimdat (entget dimnme)
         dimtxt (cdr (assoc 1 dimdat))
         bit70 (cdr (assoc 70 dimdat))
         bit70 (- bit70 (logand 32 bit70)(logand 64 bit70)(logand 128 bit70)) ; we only need the non-bitcoded portion (what a mess)
   )
   (if 
    (and
     (not (member (vlax-ename->vla-object dimnme) (vlr-owners reactor)))
     (/= 2 bit70) ; do not attach to angular dimensions
     (KHAD:check_dimlin dimnme)
     ;(/= 34 bit70)
     ;(/= 130 bit70)
     ;(/= 162 bit70)
    )
    (progn
     (command "_.DIMEDIT" "_new" "" dimnme "")
     (setq dimobj (vlax-ename->vla-object dimnme))
     (if (vlr-owner-add reactor dimobj)(setq count (1+ count)))
     (command "_.DIMEDIT" "_new" dimtxt dimnme "")
    )
   )
  )
 )
 (princ (strcat "\nattached persistent formatting to " (itoa count) " dimensions"))
 (princ)
)

;remove the persistent tolerance formatting from dimensions
(defun KHAD:remove_toleranceformat ( / reactor selset n count dimnme dimtxt dimobj)
 (if (setq reactor (KHAD:find_reactor :vlr-object-reactor "KHAD:dimtolerancereactor"))
  (progn
   (setq count 0)
   (prompt "\nSelect dimension objects to remove formatting: ")
   (setq selset (ssget '((-4 . "<XOR")(0 . "DIMENSION")(0 . "ARC_DIMENSION")(-4 . "XOR>"))))
   (repeat (setq n (sslength selset))
    (setq dimnme (ssname selset (setq n (1- n))))
    (if (member (vlax-ename->vla-object dimnme) (vlr-owners reactor))
     (progn
      (setq dimtxt (cdr (assoc 1 (entget dimnme))))
      ;(command "_.DIMEDIT" "_new" " " dimnme "")
      (setq dimobj (vlax-ename->vla-object dimnme))
      (if (vlr-owner-remove reactor dimobj)(setq count (1+ count)))
      (command "_.DIMEDIT" "_new" dimtxt dimnme "")
     )
    )
   )
   (princ (strcat "\nremoved persistent formatting from " (itoa count) " dimensions"))
  )
  (princ "\nNo reactor found")
 )
 (princ)
)


;;; reactor callback
(defun KHAD:format_tolerance (object reactor parameters / dimnme)
 (setq dimnme (vlax-vla-object->ename object))
 (if (setq dimdat (entget dimnme)) ; check if dimnme has not been erased by entdel (bcad crashes otherwise)
  (KHAD:hide_lower_tolerance dimnme)
 )
)


;;; Command definitions - edit to your needs (overrides your acad.pgp file!)
(defun C:ATF ( / )
 (KHAD:attach_toleranceformat)
)

(defun C:RTF ( / )
 (KHAD:remove_toleranceformat)
)

(defun C:STF ( / )
 (command "_.SELGRIPS" (KHAD:select_reactor_owners :vlr-object-reactor "KHAD:dimtolerancereactor") "")
)



